@@ -2,6 +2,7 @@
 
 import logging
 import struct
+import sys
 from collections import namedtuple
 import numpy as np
 
@@ -11,13 +12,18 @@ log.setLevel(logging.WARNING)
 # To adjust the log level for this module from a script, use eg:
 # logging.getLogger(tdms.__name__).setLevel(logging.DEBUG)
 
+try:
+    long
+except NameError:
+    # Python 3
+    long = int
 tocProperties = {
-    'kTocMetaData': (1L << 1),
-    'kTocRawData': (1L << 3),
-    'kTocDAQmxRawData': (1L << 7),
-    'kTocInterleavedData': (1L << 5),
-    'kTocBigEndian': (1L << 6),
-    'kTocNewObjList': (1L << 2)
+    'kTocMetaData': (long(1) << 1),
+    'kTocRawData': (long(1) << 3),
+    'kTocDAQmxRawData': (long(1) << 7),
+    'kTocInterleavedData': (long(1) << 5),
+    'kTocBigEndian': (long(1) << 6),
+    'kTocNewObjList': (long(1) << 2)
 }
 
 # Class for describing data types, with data type name,
@@ -225,14 +231,14 @@ class TdmsSegment(object):
             # If this is a new object, add it to the object dictionary,
             # otherwise reuse the existing object
             if object_path in objects:
-                object = objects[object_path]
+                obj = objects[object_path]
             else:
-                object = TdmsObject(object_path)
-                objects[object_path] = object
-            object.read_metadata(f)
+                obj = TdmsObject(object_path)
+                objects[object_path] = obj
+            obj.read_metadata(f)
             if (self.toc["kTocNewObjList"] or
                     object_path not in [o.path for o in self.ordered_objects]):
-                self.ordered_objects.append(object)
+                self.ordered_objects.append(obj)
 
     def read_raw_data(self, f, objects):
         """Read signal data from file"""
@@ -251,7 +257,7 @@ class TdmsSegment(object):
             raise ValueError("Data size %d is not a multiple of the "
                     "chunk size %d" % (total_data_size, data_size))
         else:
-            num_chunks = total_data_size / data_size
+            num_chunks = total_data_size // data_size
         log.debug("Reading %d bytes of data at %d in %d chunks" %
                 (total_data_size, f.tell(), num_chunks))
 
@@ -261,33 +267,76 @@ class TdmsSegment(object):
             endianness = '<'
 
         for chunk in range(num_chunks):
-            object_data = {}
-
             if self.toc["kTocInterleavedData"]:
                 log.debug("Data is interleaved")
-                for object in self.ordered_objects:
-                    if object.has_data:
-                        object_data[object.path] = object.new_segment_data()
-                data_objects = [
-                        o for o in self.ordered_objects
-                        if o.has_data]
-                # Format documentation doesn't state that interleaved data
-                # must have the same number of values in each channel,
-                # but we assume it must
-                for i in range(data_objects[0].number_values):
-                    for object in data_objects:
-                        object_data[object.path][i] = object.read_value(
-                                f, endianness)
+                data_objects = [o for o in self.ordered_objects if o.has_data]
+                # If all data types have numpy types and all the lengths are
+                # the same, then we can read all data at once with numpy,
+                # which is much faster
+                all_numpy = all(
+                        (o.data_type.nptype is not None for o in data_objects))
+                same_length = (len(
+                        set((o.number_values for o in data_objects))) == 1)
+                if (all_numpy and same_length):
+                    self._read_interleaved_numpy(f, data_objects, endianness)
+                else:
+                    self._read_interleaved(f, data_objects, endianness)
             else:
+                object_data = {}
                 log.debug("Data is contiguous")
-                for object in self.ordered_objects:
-                    if object.has_data:
-                        object_data[object.path] = (
-                                object.read_values(f, endianness))
+                for obj in self.ordered_objects:
+                    if obj.has_data:
+                        object_data[obj.path] = (
+                                obj.read_values(f, endianness))
 
-            for object in self.ordered_objects:
-                if object.has_data:
-                    object.update_data(object_data[object.path])
+                for obj in self.ordered_objects:
+                    if obj.has_data:
+                        obj.update_data(object_data[obj.path])
+
+    def _read_interleaved_numpy(self, f, data_objects, endianness):
+        """Read interleaved data where all channels have a numpy type"""
+
+        log.debug("Reading interleaved data all at once")
+        # Read all data into 1 byte unsigned ints first
+        all_channel_bytes = sum((o.data_type.length for o in data_objects))
+        number_bytes = all_channel_bytes * data_objects[0].number_values
+        combined_data = np.fromfile(f, dtype=np.uint8, count=number_bytes)
+        # Reshape, so that one row is all bytes for all objects
+        combined_data = combined_data.reshape(-1, all_channel_bytes)
+        # Now set arrays for each channel
+        data_pos = 0
+        for (i, obj) in enumerate(data_objects):
+            byte_columns = tuple(
+                    range(data_pos, obj.data_type.length + data_pos))
+            log.debug("Byte columns for channel %d: %s" % (i, byte_columns))
+            # Select columns for this channel, so that number of values will
+            # be number of bytes per point * number of data points
+            object_data = combined_data[:, byte_columns].ravel()
+            # Now set correct data type, so that the array length should
+            # be correct
+            object_data.dtype = (
+                    np.dtype(obj.data_type.nptype).newbyteorder(endianness))
+            obj.update_data(object_data)
+            data_pos += obj.data_type.length
+
+    def _read_interleaved(self, f, data_objects, endianness):
+        """Read interleaved data that doesn't have a numpy type"""
+
+        log.debug("Reading interleaved data point by point")
+        object_data = {}
+        points_added = {}
+        for obj in data_objects:
+            object_data[obj.path] = obj.new_segment_data()
+            points_added[obj.path] = 0
+        while any([points_added[o.path] < o.number_values
+                for o in data_objects]):
+            for obj in data_objects:
+                if points_added[obj.path] < obj.number_values:
+                    object_data[obj.path][points_added[obj.path]] = (
+                            obj.read_value(f, endianness))
+                    points_added[obj.path] += 1
+        for obj in data_objects:
+            obj.update_data(object_data[obj.path])
 
 
 class TdmsObject(object):
